#!python3
# coding: utf-8
"""
歌唱データベース用のLABファイルとデータを扱うモジュールです。
"""


def main():
    """実行されたときの挙動"""
    print('labファイル読み取り動作テストをします。')
    path_lab = input('path_lab: ')
    label = load(path_lab)
    for phoneme in label.values:
        print(phoneme.values)


def load_as_plainlist(path, mode='r', encoding='utf-8', kiritan=False):
    """
    labファイルを ふつうの2次元リストとして読み取る。
    旧バージョンの utaupy.label.load() に近い動作をする。
    """
    # labファイルを読み取り
    with open(path, mode=mode, encoding=encoding) as f:
        lines = [s.strip().split() for s in f.readlines()]
    # 入力ファイル末尾の空白行を除去
    while lines[-1] == ['']:
        del lines[-1]

    # リストにする [[開始時刻, 終了時刻, 発音], [], ...]
    if kiritan:
        # きりたんDBのモノラベル形式の場合、時刻が 0.0000000[s] なのでfloatを経由する。
        l = [[int(10000000 * float(v[0])), int(10000000 * float(v[1])), v[2]] for v in lines]
    else:
        # Sinsyのモノラベル形式の場合、時刻が 1234567[100ns] なのでintにする。
        l = [[int(v[0]), int(v[1]), v[2]] for v in lines]
    return l


def load(path, mode='r', encoding='utf-8', kiritan=False):
    """
    labファイルを読み取って Label クラスオブジェクトにする
    時刻を整数にすることに注意
    """
    # lab ファイル読み取り
    with open(path, mode=mode, encoding=encoding) as f:
        lines = [s.strip().split() for s in f.readlines()]
    # 入力ファイル末尾の空白行を除去
    while lines[-1] == ['']:
        del lines[-1]

    # リストにする [[開始時刻, 終了時刻, 発音], [], ...]
    if kiritan:
        phonemes = []
        for v in lines:
            phoneme = Phoneme()
            phoneme.start = int(10000000 * float(v[0]))
            phoneme.end = int(10000000 * float(v[1]))
            phoneme.symbol = v[2]
            phonemes.append(phoneme)

    # Sinsyのモノラベル形式の場合、時刻が 1234567[100ns] なのでintにする。
    else:
        phonemes = []
        for v in lines:
            phoneme = Phoneme()
            phoneme.start = int(v[0])
            phoneme.end = int(v[1])
            phoneme.symbol = v[2]
            phonemes.append(phoneme)

    # Labelクラスオブジェクト化
    label = Label()
    label.values = phonemes
    return label


class Label:
    """
    歌唱ラベルLABファイルを想定したクラス(2019/04/19から)
    """

    def __init__(self):
        """二次元リスト [[開始時刻, 終了時刻, 発音], [], ...]"""
        self.__phonemes = []

    @property
    def values(self):
        """propertyはgetterも兼ねるらしい"""
        return self.__phonemes

    @values.setter
    def values(self, phonemes):
        """
        値を登録
        """
        if not isinstance(phonemes, list):
            raise TypeError(
                'argument \'phonemes\' must be list instance (values.setter in utaupy.label.py)')
        self.__phonemes = phonemes

    def check_invalid_time(self, threshold=0):
        """
        発声時間が一定未満な音素ラベル行を検出
        threshold: 許容される最小の発声時間(ms)
        """
        threshold_100ns = int(threshold * (10**4))
        for i, phoneme in enumerate(self.__phonemes):
            duration = phoneme.end - phoneme.start
            if duration < threshold_100ns:
                start = phoneme.start
                end = phoneme.end
                s_now = phoneme.symbol
                s_prev = phoneme.values[i - 1].symbol
                s_next = phoneme.values[i + 1].symbol
                print(
                    f'    [ERROR] 発声時間が {threshold_100ns}ms 未満か負です : {start} {end} ({s_prev}) {s_now} ({s_next})')

    def write(self, path_out, mode='w',
              encoding='utf-8', newline='\n', delimiter=' ', kiritan=False):
        """
        LABファイルを書き出し
        """
        # 出力用の文字列
        phonemes = self.__phonemes
        if kiritan:
            lines = ['{:.7f} {:.7f} {}'.format(ph.start, ph.end, ph.symbol)
                     for ph in phonemes]  # 100ns -> 1s 表記変換
        else:
            lines = ['{0}{3}{1}{3}{2}'.format(
                ph.start, ph.end, ph.symbol, delimiter) for ph in phonemes]
        # ファイル出力
        with open(path_out, mode=mode, encoding=encoding, newline=newline) as f:
            f.write('\n'.join(lines))
        return lines


class Phoneme:
    """
    ラベルの一行分の情報を持つクラス(2020/07/23から)
    """

    def __init__(self):
        self.start = None   # 発声開始位置
        self.end = None     # 発声終了位置
        self.symbol = None  # 発音記号

    def __str__(self):
        return f'{self.start} {self.end} {self.symbol} \t<utaupy.label.Phoneme class object>'

    @property
    def values(self):
        """
        値を確認する用に一応実装
        setterは用意しない
        """
        d = {'self.start': self.start,
             'self.end': self.end,
             'self.symbol': self.symbol}
        return d


if __name__ == '__main__':
    main()

if __name__ == '__init__':
    pass



#!python3
# coding: utf-8
# Copyright (c) oatsu
"""
UTAUのプラグイン用のモジュール
基本的には utaupy.ust の Ust() とか Note() を流用する。

【注意】本スクリプトは開発初期なため仕様変更が激しいです。
"""

from copy import deepcopy
from pprint import pprint
from sys import argv

from utaupy import ust as _ust


def main():
    """
    直接実行されたときのやつ
    """
    print('UTAUプラグイン一時ファイルの読み取りテストをします。')
    path = input('テキストファイルのパスを入力してください。\n>>> ')
    plugin = load(path)
    for note in plugin.values:
        pprint(note.values, width=200)


def run(your_function):
    """
    UTAUプラグインスクリプトファイルの入出力をする。
    """
    # UTAUから出力されるプラグインスクリプトのパスを取得
    path = argv[1]
    # up.utauplugin.Plugin オブジェクトとしてプラグインスクリプトを読み取る
    plugin = load(path)
    # 目的のノート処理を実行
    your_function(plugin)
    # プラグインスクリプトを上書き
    plugin.write(path)


def load(path, mode='r', encoding='shift-jis'):
    """
    UTAUプラグイン一時ファイルを読み取る
    USTのやつを一部改変
    """
    ust = _ust.load(path, mode=mode, encoding=encoding)
    notes = ust.values
    # UtauPluginオブジェクト化
    plugin = UtauPlugin()
    plugin.version = notes.pop(0)
    plugin.setting = notes.pop(0)
    if notes[0].tag == '[#PREV]':
        plugin.prev = notes.pop(0)
    if notes[-1].tag == '[#NEXT]':
        plugin.next = notes.pop(-1)
    plugin.notes = notes
    return plugin


class UtauPlugin(_ust.Ust):
    """
    UTAUプラグインの一時ファイル用のクラス
    UST用のクラスを継承
    """

    def __init__(self):
        super().__init__()  # self._note = []
        self.version = None  # [#VERSION]
        self.setting = None  # [#SETTING]
        self.prev = None  # [#PREV] のNoteオブジェクト
        self.next = None  # [#NEXT] のNoteオブジェクト

    @property
    def notes(self):
        return self._notes

    @notes.setter
    def notes(self, l):
        self._notes = l

    def write(self, path, mode='w', encoding='shift-jis'):
        """
        プラグイン用のテキストをファイル出力する。
        UST と違って[#DELETE]でも書き込む。
        """
        duplicated_self = deepcopy(self)
        lines = []
        for note in duplicated_self.values:
            # ノートを解体して行のリストにする
            d = note.values
            lines.append(d.pop('Tag'))
            for k, v in d.items():
                lines.append('{}={}'.format(str(k), str(v)))
        # 出力用の文字列
        s = '\n'.join(lines)
        # ファイル出力
        with open(path, mode=mode, encoding=encoding) as f:
            f.write(s)
        return s


if __name__ == '__main__':
    main()



#!python
# coding: utf-8
# Copyright (c) oatsu
"""
入出力ファイルのバックアップを取りたいときに使う
"""

from datetime import datetime
from os import path, makedirs
from shutil import copy2


def backup_io(path_file, outdirname):
    """
    ファイル名に時刻を組み込んでバックアップ
    入出力ファイル向け。
    """
    # バックアップ用のフォルダを生成
    makedirs(f'backup/{outdirname}', exist_ok=True)

    basename, ext = path.splitext(path.basename(path_file))
    now = datetime.now().strftime('%Y%m%d_%H%M%S')

    copy2(path_file, f'backup/{outdirname}/{basename}__{now}{ext}')



#!python3
# coding: utf-8
"""
日本語とアルファベットの対応表を扱うモジュールです。
"""


def main():
    """呼び出されても特に何もしない"""
    print('平仮名とアルファベットの対応表を扱うモジュールです。')


def load(path):
    """テーブルを読み取ってインスタンス生成"""
    # ファイル読み取り
    with open(path) as f:
        l = [v.strip().split() for v in f.readlines()]
    # 辞書にする
    d = {}
    for v in l:
        d[v[0]] = v[1:]
    return d


if __name__ == '__main__':
    main()

if __name__ == '__init__':
    print('utaupy.table imported')



#!python3
# coding: utf-8
"""
setParam用のINIファイルとデータを扱うモジュールです。
"""
import re

# from . import table


def main():
    """実行されたときの挙動"""
    print('耳ロボPとsetParamに卍感謝卍')


def load(path, mode='r', encoding='shift-jis'):
    """otoiniを読み取ってオブジェクト生成"""
    # otoiniファイルを読み取る
    with open(path, mode=mode, encoding=encoding) as f:
        l = [re.split('[=,]', s.strip()) for s in f.readlines()]
    # # 入力ファイル末尾の空白行を除去
    # while l[-1] == ['']:
    #     del l[-1]

    # Otoクラスオブジェクトのリストを作る
    otolist = []
    for v in l:
        oto = Oto()
        oto.from_otoini(v)
        otolist.append(oto)
    # OtoIniクラスオブジェクト化
    o = OtoIni()
    o.values = otolist
    return o


class OtoIni:
    """oto.iniを想定したクラス"""

    def __init__(self):
        # 'Oto'クラスからなるリスト
        self.__values = []

    @property
    def values(self):
        """中身を確認する"""
        return self.__values

    @values.setter
    def values(self, list_of_oto):
        """中身を上書きする"""
        self.__values = list_of_oto

    def replace_aliases(self, before, after):
        """エイリアスを置換する"""
        for oto in self.__values:
            oto.alias = oto.alias.replace(before, after)
        return self

    def is_mono(self):
        """
        モノフォン形式のエイリアスになっているか判定する。
        返り値はbool。
        """
        return all(len(v.alias.split()) == 1 for v in self.__values)

    def as_dict(self):
        """
        辞書に変換して返す。
        エイリアスが重複していると古いほうは消される。
        """
        d = {oto.alias: oto for oto in self.__values}
        return d

    def kana2romaji(self, d_table):
        """
        エイリアスをローマ字にする
        replace:
          Trueのときエイリアスをローマ字に書き換え
          Falseのときエイリアスは平仮名のまま
        """
        for oto in self.__values:
            try:
                oto.alias = ' '.join(d_table[oto.alias])
            except KeyError as e:
                print(f'[ERROR] KeyError in utaupy.otoini.OtoIni.kana2romaji: {e}')

    def monophonize(self):
        """
        音素ごとに分割する。
        otoini→label 変換の用途を想定
        音素の発声開始位置: 左ブランク=先行発声
        """
        # 新規OtoIniを作るために、otoを入れるリスト
        l = []
        for oto in self.__values:
            phonemes = oto.alias.split()
            if len(phonemes) == 1:
                l.append(oto)
            elif len(phonemes) in [2, 3]:
                name_wav = oto.filename
                # 1文字目(オーバーラップから先行発声まで)------------
                mono_oto = Oto()
                mono_oto.filename = name_wav
                mono_oto.alias = phonemes[0]
                mono_oto.offset = oto.offset + oto.overlap  # オーバーラップの位置に左ブランクを移動
                mono_oto.preutterance = 0
                l.append(mono_oto)
                # 2文字目(先行発声から固定範囲まで)----------------
                mono_oto = Oto()
                mono_oto.filename = name_wav
                mono_oto.alias = phonemes[1]
                mono_oto.offset = oto.offset + oto.preutterance  # 先行発声の位置に左ブランクを移動
                mono_oto.preutterance = 0
                l.append(mono_oto)
                if len(phonemes) == 3:
                    # 3文字目(固定範囲から右ブランクまで)----------------
                    mono_oto = Oto()
                    mono_oto.filename = name_wav
                    mono_oto.alias = phonemes[2]
                    mono_oto.offset = oto.offset + oto.consonant  # 固定範囲の位置に左ブランクを移動
                    mono_oto.preutterance = 0
                    l.append(mono_oto)
            else:
                print('\n[ERROR in otoini.monophonize()]----------------')
                print('  エイリアスの音素数は 1, 2, 3 以外対応していません。')
                print('  phonemes: {}'.format(phonemes))
                print('  文字を連結して処理を続行します。')
                print('-----------------------------------------------\n')
                l.append(oto)
        self.__values = l

    def write(self, path, mode='w', encoding='shift-jis'):
        """OtoIniクラスオブジェクトをINIファイルに出力"""
        s = ''
        for oto in self.__values:
            l = []
            l.append(oto.filename)
            l.append(oto.alias)
            l.append(oto.offset)
            l.append(oto.consonant)
            l.append(oto.cutoff)
            l.append(oto.preutterance)
            l.append(oto.overlap)
            # 数値部分を丸めてから文字列に変換
            l = l[:2] + [str(round(float(v), 4)) for v in l[2:]]
            s += '{}={},{},{},{},{},{}\n'.format(*l)  # 'l[0]=l[1],l[2],...'
        with open(path, mode=mode, encoding=encoding) as f:
            f.write(s)
        return s


class Oto:
    """oto.ini中の1モーラ"""

    def __init__(self):
        keys = ('FileName', 'Alias', 'Offset',
                'Consonant', 'Cutoff', 'Preutterance', 'Overlap')
        tpl = (None, None, 0, 0, 0, 0, 0, 0)
        self.__d = dict(zip(keys, tpl))

    def __str__(self):
        return f'\'{self.alias}\'\t<utaupy.otoini.Oto object at {id(self)}>'

    def from_otoini(self, l):
        """1音分のリストをもらってクラスオブジェクトにする"""
        keys = ('FileName', 'Alias', 'Offset',
                'Consonant', 'Cutoff', 'Preutterance', 'Overlap')
        # 数値部分をfloatにする
        l = l[:2] + [float(v) for v in l[2:]]
        self.__d = dict(zip(keys, l))
        return self

    # ここからノートの全値の処理----------------------
    @property
    def values(self):
        """中身を確認する"""
        return self.__d

    @values.setter
    def values(self, d):
        """中身を上書きする"""
        self.__d = d
    # ここまでノートの全値の処理----------------------

    # ここからノートの各値の処理----------------------
    @property
    def filename(self):
        """wavファイル名を確認する"""
        return self.__d['FileName']

    @filename.setter
    def filename(self, x):
        """wavファイル名を上書きする"""
        self.__d['FileName'] = x

    @property
    def alias(self):
        """エイリアスを確認する"""
        return self.__d['Alias']

    @alias.setter
    def alias(self, x):
        """エイリアスを上書きする"""
        self.__d['Alias'] = x

    @property
    def offset(self):
        """左ブランクを確認する"""
        return self.__d['Offset']

    @offset.setter
    def offset(self, x):
        """左ブランクを上書きする"""
        self.__d['Offset'] = x

    @property
    def offset2(self):
        """左ブランクの値を取得する"""
        return self.__d['Offset']

    @offset2.setter
    def offset2(self, x):
        """
        左ブランクの値を上書きする
        左ブランクが変化した分だけほかのパラメータの数値を調整するため、
        ほかの値の絶対時刻を変化させず、左ブランクだけを移動できる。
        """
        original_offset = self.__d['Offset']
        self.__d['Offset'] = x
        dt = self.__d['Offset'] - original_offset
        self.__d['Overlap'] += dt
        self.__d['Preutterance'] += dt
        self.__d['Consonant'] += dt
        self.cutoff2 += dt

    @property
    def consonant(self):
        """固定範囲を確認する"""
        return self.__d['Consonant']

    @consonant.setter
    def consonant(self, x):
        """固定範囲を上書きする"""
        self.__d['Consonant'] = x

    @property
    def cutoff(self):
        """右ブランクを確認する"""
        return self.__d['Cutoff']

    @cutoff.setter
    def cutoff(self, x):
        """右ブランクを上書きする"""
        self.__d['Cutoff'] = x

    @property
    def cutoff2(self):
        """右ブランクを絶対時刻で取得する"""
        return max(self.__d['Cutoff'], self.__d['Offset'] - self.__d['Cutoff'])

    # OffsetがNullのとき処理できない
    @cutoff2.setter
    def cutoff2(self, x):
        """右ブランクを絶対時刻で受け取り、負の値で上書きする"""
        self.__d['Cutoff'] = self.__d['Offset'] - x

    @property
    def preutterance(self):
        """先行発声を取得する"""
        return self.__d['Preutterance']

    @preutterance.setter
    def preutterance(self, x):
        """先行発声を上書きする"""
        self.__d['Preutterance'] = x

    @property
    def overlap(self):
        """オーバーラップを取得する"""
        return self.__d['Overlap']

    @overlap.setter
    def overlap(self, x):
        """オーバーラップを上書きする"""
        self.__d['Overlap'] = x
    # ここまでノートの各値の処理----------------------


if __name__ == '__main__':
    main()

if __name__ == '__init__':
    pass



#!/usr/bin/env python3
# coding: utf-8
"""
UTAU関連ファイルの相互変換
"""
# from pysnooper import snoop
# from pprint import pprint


# from . import reaper as _reaper
# from . import reclist as _reclist
# from . import table as _table
from . import label as _label
from . import otoini as _otoini
from . import ust as _ust


def main():
    """ここ書く必要なくない？"""
    print('AtomとReaperが好き')


def svp2ust(svp, debug=False):
    """
    svpファイルを受け取って、簡易的なustオブジェクトにする。
    """
    svnotes = svp['tracks'][0]['mainGroup']['notes']

    # ust.Noteを入れておくリスト
    l = []
    # バージョン情報の空ノートを追加
    utaunote = _ust.Note()
    l.append(utaunote)
    # プロジェクト設定のノートを追加
    utaunote = _ust.Note()
    utaunote.set_by_key('Tempo', svp['time']['tempo'][0]['bpm'])
    l.append(utaunote)

    # 前奏の休符を追加
    utaunote = _ust.Note()
    utaunote.lyric = 'R'
    utaunote.length = svnotes[0]['onset'] // 1470000
    l.append(utaunote)

    # DEBUG: 休符が挟まってるかどうかを判定して、休符を追加する処理を実装する必要がある。
    for svnote in svnotes:
        utaunote = _ust.Note()
        utaunote.lyric = svnote['lyrics']
        utaunote.length = svnote['duration'] // 1470000
        l.append(utaunote)
        if debug:
            print(utaunote.values)

    ust = _ust.Ust()
    ust.values = l
    return ust


def ust2otoini(ust, name_wav, d_table, mode='romaji_cv', dt=100, replace=True, debug=False):
    """
    UstクラスオブジェクトからOtoIniクラスオブジェクトを生成
    機能選択部分
    """
    allowed_modes = ['mono', 'romaji_cv']
    if mode == 'romaji_cv':
        print('  変換モード : ひらがな歌詞 → ローマ字CV')
        otoini = ust2otoini_romaji_cv(ust, name_wav, d_table, dt, replace=replace, debug=debug)
    elif mode == 'mono':
        print('  変換モード : ひらがな歌詞 → ローマ字モノフォン')
        otoini = ust2otoini_mono(ust, name_wav, d_table, debug=debug)
    else:
        raise ValueError('argument \'mode\' must be in {}'.format(allowed_modes))
    return otoini


def ust2otoini_mono(ust, name_wav, d_table, dt=100, debug=False):
    """
    UstクラスオブジェクトからOtoIniクラスオブジェクトを生成
    vowel_otoの対称は母音以外に 'N', 'cl' なども含まれる。
    mode : otoiniのエイリアス種別選択
    【パラメータ設定図】
    simple_oto---------------------------------------------------------------
      |||左ブランク・オーバーラップ・先行発声 ||固定範囲・右ブランク
      |||       ノート長 + 2dt (ms)           ||
    -------------------------------------------------------------------------
    子音・半母音用のoto------------------------------------------------------
      ||左ブランク・オーバーラップ |先行発声 ||固定範囲・右ブランク
      ||          dt(ms)           | dt(ms)  ||
    -------------------------------------------------------------------------
    母音用のoto--------------------------------------------------------------
      ||左ブランク・オーバーラップ |先行発声 |固定範囲            |右ブランク
      ||          dt(ms)           | dt(ms)  | ノート長 - dt (ms) |
    -------------------------------------------------------------------------
    """
    ust.make_finalnote_R()  # 最終ノートが休符じゃない場合を対策
    notes = ust.values

    # UstのNoteオブジェクトごとにOtoオブジェクトを生成
    l = []  # simple_otoを入れるリスト
    t = 0  # ノート開始時刻を記録
    for note in notes[2:-1]:
        length = note.length_ms
        simple_oto = _otoini.Oto()  # 各パラメータ位置を両端に集めたOto
        simple_oto.filename = name_wav
        simple_oto.alias = note.lyric
        simple_oto.offset = t
        simple_oto.overlap = 0
        simple_oto.preutterance = 0
        simple_oto.consonant = length
        simple_oto.cutoff = -length  # 負で左ブランク相対時刻, 正で絶対時刻
        l.append(simple_oto)
        t += length  # 今のノート終了位置が次のノート開始位置

    # Otoを音素ごとに分割
    new = []  # mono_otoを入れるリスト
    for simple_oto in l:
        if debug:
            print(f'    {simple_oto.values}')
        try:
            phonemes = d_table[simple_oto.alias]
        except KeyError as e:
            print('KeyError in utaupy.convert.ust2otoini_mono---------')
            print('ひらがなローマ字変換に失敗しました。半角スペースで音素分割してぶち込みます。')
            print('エラー詳細:', e)
            print('--------------------------------------\n')
            phonemes = simple_oto.alias.split()
            print(phonemes)
        # 子音+母音 「か(k a)」
        if len(phonemes) == 2:
            # 子音部分
            first_oto = _otoini.Oto()
            first_oto.filename = name_wav
            first_oto.alias = phonemes[0]
            first_oto.offset = simple_oto.offset - (2 * dt)
            first_oto.overlap = 0
            first_oto.preutterance = dt
            first_oto.consonant = 2 * dt
            first_oto.cutoff = - 2 * dt
            new.append(first_oto)
            # 母音部分
            second_oto = _otoini.Oto()
            second_oto.filename = name_wav
            second_oto.alias = phonemes[1]
            second_oto.offset = simple_oto.offset - dt
            second_oto.overlap = 0
            second_oto.preutterance = dt
            second_oto.consonant = 2 * dt
            second_oto.cutoff = simple_oto.cutoff - dt
            new.append(second_oto)
        # 母音など 「あ(a)」「ん(cl)」「っ(cl)」「(pau)」「(br)」「(sil)」
        elif len(phonemes) == 1:
            first_oto = _otoini.Oto()
            first_oto.filename = name_wav
            first_oto.alias = phonemes[0]
            first_oto.offset = simple_oto.offset - dt
            first_oto.overlap = 0
            first_oto.preutterance = dt
            first_oto.consonant = 2 * dt
            first_oto.cutoff = simple_oto.cutoff - dt
            new.append(first_oto)
        # 子音+半母音+母音 「ぐぁ(g w a)」
        elif len(phonemes) == 3:
            # 子音部分
            first_oto = _otoini.Oto()
            first_oto.filename = name_wav
            first_oto.alias = phonemes[0]
            first_oto.offset = simple_oto.offset - (2 * dt)
            first_oto.overlap = 0
            first_oto.preutterance = dt
            first_oto.consonant = 2 * dt
            first_oto.cutoff = - 2 * dt
            new.append(first_oto)
            # 半母音部分
            second_oto = _otoini.Oto()
            second_oto.filename = name_wav
            second_oto.alias = phonemes[1]
            second_oto.offset = simple_oto.offset - dt
            second_oto.overlap = 0
            second_oto.preutterance = dt
            second_oto.consonant = 2 * dt
            second_oto.cutoff = - 2 * dt
            new.append(second_oto)
            # 母音部分
            third_oto = _otoini.Oto()
            third_oto.filename = name_wav
            third_oto.alias = phonemes[2]
            third_oto.offset = simple_oto.offset
            third_oto.overlap = 0
            third_oto.preutterance = dt
            third_oto.consonant = 2 * dt
            third_oto.cutoff = simple_oto.cutoff - (2 * dt)
            new.append(third_oto)
        else:
            raise ValueError('len(alias) must be in [1, 2, 3]')

    new[0].offset = 0
    new[0].preutterance = 0
    mono_otoini = _otoini.OtoIni()
    mono_otoini.values = new
    return mono_otoini


def ust2otoini_romaji_cv(ust, name_wav, d_table, dt=100, replace=True, debug=False):
    """
    UstクラスオブジェクトからOtoIniクラスオブジェクトを生成
    dt   : 左ブランク - オーバーラップ - 先行発声 - 固定範囲と右ブランク の距離
    mode : otoiniのエイリアス種別選択
    【パラメータ設定図】
      | 左ブランク |オーバーラップ| 先行発声 | 固定範囲 |   右ブランク   |
      |   (dt)ms   |    (dt)ms    |  (dt)ms  |  (dt)ms  | (length-2dt)ms |
    """
    ust.make_finalnote_R()  # 最終ノートが休符じゃない場合を対策
    notes = ust.values
    l = []  # otoini生成元にするリスト
    t = 0  # ノート開始時刻を記録

    # NOTE: ここnotes[2:-1]とust.values[2:-1]で処理時間に差は出る？
    for note in notes[2:-1]:
        if debug:
            print(f'    {note.values}')
        try:
            phonemes = d_table[note.lyric]
        except KeyError as e:
            print(f'    [ERROR] KeyError in utaupy.convert.ust2otoini_romaji_cv : {e}')
            phonemes = note.lyric.split()

        length = note.length_ms
        oto = _otoini.Oto()
        oto.filename = name_wav     # wavファイル名
        if replace:
            oto.alias = ' '.join(phonemes)  # エイリアスは音素ごとに空白区切り
        else:
            oto.alias = note.lyric
        oto.offset = t - (2 * dt)   # 左ブランクはノート開始位置より2段手前
        oto.preutterance = 2 * dt   # 先行発声はノート開始位置
        oto.consonant = min(3 * dt, length + 2 * dt)  # 子音部固定範囲は先行発声より1段後ろか終端
        oto.cutoff = -(length + 2 * dt)  # 右ブランクはノート終端、負で左ブランク相対時刻、正で絶対時刻

        # 1音素のときはノート開始位置に先行発声を配置
        if len(phonemes) == 1:
            oto.overlap = 0

        # 2,3音素の時はノート開始位置に先行発声、その手前にオーバーラップ
        elif len(phonemes) in (2, 3):
            oto.overlap = dt

        # 4音素以上には未対応。特殊音素と判断して1音素として処理
        else:
            print('\nERROR when setting alias : phonemes = {}-------------'.format(phonemes))
            print('1エイリアスあたり 1, 2, 3 音素しか対応していません。')
            oto.alias = ''.join(phonemes)
            oto.overlap = 0

        l.append(oto)
        t += length  # 今のノート終了位置が次のノート開始位置

    # 最初が休符なことを想定して、
    l[0].offset = 0  # 最初の左ブランクを0にする
    l[0].overlap = 0  # 最初のオーバーラップを0にする
    l[0].preutterance = 0  # 最初の先行発声を0にする
    l[0].cutoff2 -= 2 * dt
    otoini = _otoini.OtoIni()
    otoini.values = l
    return otoini


def otoini2label(otoini, mode='auto', debug=False):
    """
    OtoIniクラスオブジェクトからLabelクラスオブジェクトを生成
    発声開始: オーバーラップ
    発声終了: 次のノートのオーバーラップ
    発音記号: エイリアス流用
    otoini_time_order: otoiniの時間単位の桁。
    label_time_order : ラベルの時間単位の桁。
    """
    time_order_ratio = 10000
    # time_order_ratio = otoini_time_order / label_time_order
    # print('  time_order_ratio:', time_order_ratio)

    allowed_modes = ['auto', 'mono', 'romaji_cv']
    # エイリアスのタイプ自動判別
    if mode == 'auto':
        if otoini.is_mono():
            mode = 'mono'
        else:
            mode = 'romaji_cv'
    if mode == 'romaji_cv':
        print('  mode: OtoIni(romaji_cv) -> Label(mono)')
        # モノフォン化
        otoini.monophonize()
    elif mode == 'mono':
        print('  mode: OtoIni(mono) -> Label(mono)')
    else:
        raise ValueError('argument \'mode\' must be in {}'.format(allowed_modes))

    # 計算の重複を避けるために [[発音開始時刻, 発音記号], ...] の仮リストにする
    tmp = []
    otoini_values = otoini.values
    for oto in otoini_values:
        if debug:
            print(f'    {oto.values}')
        t_start = int(time_order_ratio * (oto.offset + oto.preutterance))
        tmp.append([t_start, oto.alias])

    # OtoオブジェクトをPhonemeオブジェクトに変換する
    phonemes = []
    for i, v in enumerate(tmp[:-1]):
        phoneme = _label.Phoneme()
        phoneme.start = v[0]
        phoneme.end = tmp[i + 1][0]
        phoneme.symbol = v[1]
        phonemes.append(phoneme)

    # 最終Otoだけ終了位置が必要なので 特別な処理
    oto = otoini_values[-1]
    phoneme = _label.Phoneme()
    phoneme.start = int(time_order_ratio * (oto.offset + oto.preutterance))
    phoneme.end = int(time_order_ratio * oto.cutoff2)  # 発声終了位置は右ブランク
    phoneme.symbol = tmp[-1][1]
    phonemes.append(phoneme)

    # Labelクラスオブジェクト化
    label = _label.Label()
    label.values = phonemes
    return label


def label2otoini(label, name_wav):
    """
    LabelオブジェクトをOtoIniオブジェクトに変換
    モノフォン、CV、VCV とかの選択肢が必要そう
    otoini_time_order: otoiniの時間オーダー。
    label_time_order : ラベルの時間オーダー。
    """
    # time_order_ratio = label_time_order / otoini_time_order
    time_order_ratio = 10**(-4)

    l = []  # Otoオブジェクトを格納するリスト

    # 各音素PhonemeオブジェクトをOtoオブジェクトに変換して、リストにまとめる。
    for phoneme in label.values:
        oto = _otoini.Oto()
        oto.filename = name_wav
        oto.alias = phoneme.symbol
        oto.offset = phoneme.start * time_order_ratio
        oto.overlap = 0.0
        oto.preutterance = 0.0
        oto.consonant = (phoneme.end - phoneme.start) * time_order_ratio
        oto.cutoff = - (phoneme.end - phoneme.start) * time_order_ratio
        l.append(oto)

    # Otoiniオブジェクト化
    otoini = _otoini.OtoIni()
    otoini.values = l
    return otoini


if __name__ == '__main__':
    main()

if __name__ == '__init__':
    pass



#!python3
# coding: utf-8
"""
USTファイルとデータを扱うモジュールです。
"""
import re
from copy import deepcopy


def main():
    """実行されたときの挙動"""
    print('デフォ子かわいいよデフォ子\n')

    print('ust読み取りテストをします。')
    path = input('ustのパスを入力してください。\n>>> ')
    ust = load(path)
    print()

    for note in ust.values:
        print(note.values)

    input('\nPress Enter to exit.')


def notenum_as_abc(notenum):
    """
    音階番号をABC表記に変更する(C1=24, C4=)
    """
    d = {'24': 'C1', '25': 'C#1', '26': 'D1', '27': 'D#1', '28': 'E1', '29': 'F1',
         '30': 'F#1', '31': 'G1', '32': 'G#1', '33': 'A1', '34': 'A#1', '35': 'B1',
         '36': 'C2', '37': 'C#2', '38': 'D2', '39': 'D#2', '40': 'E2', '41': 'F2',
         '42': 'F#2', '43': 'G2', '44': 'G#2', '45': 'A2', '46': 'A#2', '47': 'B2',
         '48': 'C3', '49': 'C#3', '50': 'D3', '51': 'D#3', '52': 'E3', '53': 'F3',
         '54': 'F#3', '55': 'G3', '56': 'G#3', '57': 'A3', '58': 'A#3', '59': 'B3',
         '60': 'C4', '61': 'C#4', '62': 'D4', '63': 'D#4', '64': 'E4', '65': 'F4',
         '66': 'F#4', '67': 'G4', '68': 'G#4', '69': 'A4', '70': 'A#4', '71': 'B4',
         '72': 'C5', '73': 'C#5', '74': 'D5', '75': 'D#5', '76': 'E5', '77': 'F5',
         '78': 'F#5', '79': 'G5', '80': 'G#5', '81': 'A5', '82': 'A#5', '83': 'B5',
         '84': 'C6', '85': 'C#6', '86': 'D6', '87': 'D#6', '88': 'E6', '89': 'F6',
         '90': 'F#6', '91': 'G6', '92': 'G#6', '93': 'A6', '94': 'A#6', '95': 'B6',
         '96': 'C7', '97': 'C#7', '98': 'D7', '99': 'D#7', '100': 'E7', '101': 'F7',
         '102': 'F#7', '103': 'G7', '104': 'G#7', '105': 'A7', '106': 'A#7', '107': 'B7',
         '108': 'C8', '109': 'C#8', '110': 'D8', '111': 'D#8', '112': 'E8', '113': 'F8',
         '114': 'F#8', '115': 'G8', '116': 'G#8', '117': 'A8', '118': 'A#8', '119': 'B8'}
    if isinstance(notenum, str):
        return d[notenum]
    if isinstance(notenum, int):
        return d[str(notenum)]
    if isinstance(notenum, (list, tuple)):
        return [d[str(v)] for v in notenum]
    raise TypeError("argument 'notenum' must be in [str, int, list, tuple]")


def load(path, mode='r', encoding='shift-jis'):
    """USTを読み取り"""
    # USTを文字列として取得
    try:
        with open(path, mode=mode, encoding=encoding) as f:
            s = f.read()
    except UnicodeDecodeError:
        with open(path, mode=mode, encoding='utf-8_sig') as f:
            s = f.read()
    # USTをノート単位に分割
    l = [r'[#' + v.strip() for v in s.split(r'[#')][1:]
    # さらに行ごとに分割して二次元リストに
    l = [v.split('\n') for v in l]

    # ノートのリストを作る
    notes = []
    for lines in l:
        note = Note()
        # ノートの種類
        tag = lines[0]
        note.tag = tag
        # print('Making "Note" instance from UST: {}'.format(tag))
        # タグ以外の行の処理
        if tag == '[#VERSION]':
            note.set_by_key('Version', lines[1])
        elif tag == '[#TRACKEND]':
            pass
        else:
            for line in lines[1:]:
                key, value = line.split('=', 1)
                note.set_by_key(key, value)
        notes.append(note)

    # 旧形式の場合にタグの数を合わせる
    if notes[0].tag != r'[#VERSION]':
        version = notes[0].get_by_key('UstVersion')
        note = Note()
        note.tag = '[#VERSION]'
        note.set_by_key('UstVersion', version)
        notes.insert(0, note)  # リスト先頭に挿入
    # Ustクラスオブジェクト化
    u = Ust()
    u.values = notes
    u.version = notes[0]
    u.setting = notes[1]
    # 隠しパラメータ alternative_tempo を全ノートに設定
    u.reload_tempo()
    return u


class Ust:
    """UST"""

    def __init__(self):
        """インスタンス作成"""
        # ノート(クラスオブジェクト)からなるリスト
        self._notes = []
        self.version = None
        self.setting = None

    def __len__(self):
        return len(self._notes)

    @property
    def values(self):
        """中身を見る"""
        return self._notes

    @values.setter
    def values(self, l):
        """
        中身を上書きする
        テンポを正常に取得できるようにする
        """
        if not isinstance(l, list):
            raise TypeError('argument \"l\" must be list instance')
        self._notes = l
        self.reload_tempo()
        return self

    @property
    def notes(self):
        """
        全セクションのうち、VERSION と SETTING TRACKEND を除いたノート部分を取得
        """
        return self._notes[2:-1]

    @notes.setter
    def notes(self, l):
        """
        全セクションのうち、VERSION と SETTING TRACKEND を除いたノート部分を上書き
        """
        if not isinstance(l, list):
            raise TypeError('argument \"l\" must be list instance')
        self._notes = self._notes[:2] + l + self._notes[-1:]
        self.reload_tempo()
        return self

    @property
    def tempo(self):
        """全体のBPMを見る"""
        try:
            project_tempo = self._notes[1].tempo
            return project_tempo
        except KeyError:
            first_note_tempo = self._notes[2].tempo
            return first_note_tempo

        print('[ERROR]--------------------------------------------------')
        print('USTのテンポが設定されていません。とりあえず120にします。')
        print('---------------------------------------------------------\n')
        return '120'

    @tempo.setter
    def tempo(self, tempo):
        """
        グローバルBPMを上書きする
        """
        self._notes[1].tempo = tempo
        self._notes[2].tempo = tempo
        self.reload_tempo()

    def reload_tempo(self):
        """
        各ノートでBPMが取得できるように
        独自パラメータ note.alternative_tempo を全ノートに仕込む
        """
        current_tempo = self.tempo
        for note in self._notes[2:-1]:
            try:
                current_tempo = note.get_by_key('Tempo')
            except KeyError:
                pass
            note.alternative_tempo = float(current_tempo)

    # ノート一括編集系関数ここから----------------------------------------------
    def replace_lyrics(self, before, after):
        """歌詞を一括置換（文字列指定・破壊的処理）"""
        for note in self._notes[2:-1]:
            note.lyric = note.lyric.replace(before, after)

    def translate_lyrics(self, before, after):
        """歌詞を一括置換（複数文字指定・破壊的処理）"""
        for note in self._notes[2:-1]:
            note.lyric = note.lyric.translate(before, after)

    def vcv2cv(self):
        """歌詞を平仮名連続音から単独音にする"""
        for note in self._notes[2:-1]:
            note.lyric = note.lyric.split()[-1]
    # ノート一括編集系関数ここまで----------------------------------------------

    def insert_note(self, i):
        """
        i 番目の区切りに新規ノートを挿入する。
        このときの i は音符のみのインデックス。
        編集するために、挿入したノートを返す。
        """
        note = Note()
        note.tag = '[#INSERT]'
        self.notes.insert(i, note)
        return note

    def delete_note(self, i):
        """
        i 番目のノートを [#DELETE] する。
        """
        self.notes[i].tag = '[#DELETE]'

    def make_finalnote_R(self):
        """Ustの最後のノートが必ず休符 になるようにする"""
        note = self._notes[-2]
        # Ust内の最後はTRACKENDなので後ろから2番目のノートで判定
        if note.lyric not in ('pau', 'sil', 'R'):
            print('  末尾に休符を自動追加しました。')
            extra_note = deepcopy(note)
            extra_note.lyric = 'R'
            extra_note.alternative_tempo = note.tempo
            self._notes.insert(-1, extra_note)

    def write(self, path, mode='w', encoding='shift-jis'):
        """
        USTをファイル出力
        """
        duplicated_self = deepcopy(self)
        lines = []
        for note in duplicated_self.values:
            # ノートを解体して行のリストにする
            d = note.values
            # DELETE なノートはファイル出力しない
            if d['Tag'] == '[#DELETE]':
                continue
            lines.append(d.pop('Tag'))
            for k, v in d.items():
                lines.append(f'{str(k)}={str(v)}')
        # 出力用の文字列
        s = '\n'.join(lines)
        # ファイル出力
        with open(path, mode=mode, encoding=encoding) as f:
            f.write(s)
        return s


class Note:
    """UST内のノート"""

    def __init__(self, tag='[#UNDEFINED]'):
        self.__d = {}
        self.tag = tag
        self.alternative_tempo = 120

    def __str__(self):
        try:
            return f'{self.tag} {self.lyric}\t<utaupy.ust.Note object>'
        except KeyError:
            return f'{self.tag} {None}\t<utaupy.ust.Note object>'

    @property
    def values(self):
        """ノートの中身を見る"""
        return self.__d

    @values.setter
    def values(self, d):
        """
        ノートの中身を上書き
        """
        if not isinstance(d, dict):
            raise TypeError('argument \"d\" must be dictionary instance')
        self.__d = d

    @property
    def tag(self):
        """タグを確認"""
        return self.__d['Tag']

    @tag.setter
    def tag(self, s):
        """タグを上書き"""
        self.__d['Tag'] = s

    @property
    def length(self):
        """ノート長を確認[Ticks]"""
        return int(self.__d['Length'])

    @length.setter
    def length(self, x):
        """ノート長を上書き[Ticks]"""
        self.__d['Length'] = str(x)

    @property
    def length_ms(self):
        """ノート長を確認[ms]"""
        return 125 * float(self.__d['Length']) / self.tempo

    @length_ms.setter
    def length_ms(self, x):
        """ノート長を上書き[ms]"""
        self.__d['Length'] = x * self.tempo // 125

    @property
    def lyric(self):
        """歌詞を確認"""
        return self.__d['Lyric']

    @lyric.setter
    def lyric(self, x):
        """歌詞を上書き"""
        self.__d['Lyric'] = x

    @property
    def notenum(self):
        """音階番号を確認"""
        return int(self.__d['NoteNum'])

    @notenum.setter
    def notenum(self, x):
        """音階番号を上書き"""
        self.__d['NoteNum'] = str(x)

    @property
    def tempo(self):
        """ローカルBPMを取得"""
        try:
            return float(self.__d['Tempo'])
        except KeyError:
            return float(self.alternative_tempo)

    @tempo.setter
    def tempo(self, x):
        """BPMを上書き"""
        self.__d['Tempo'] = x

    @property
    def pbs(self):
        """
        PBS (mode2ピッチ開始位置[ms]) を取得
        例) PBS=-104;20.0
        """
        # 辞書には文字列で登録してある
        str_pbs = self.__d['PBS']
        # 浮動小数のリストに変換
        list_pbs = list(map(float, re.split('[;,]', str_pbs)))
        # PBSの値をリストで返す
        return list_pbs

    @pbs.setter
    def pbs(self, list_pbs):
        """
        PBS (mode2ピッチ開始位置[ms]) を登録
        例) PBS=-104;20.0
        """
        s1 = f'{int(list_pbs[0])};'
        s2 = ','.join(map(str, list_pbs[1:]))

        str_pbs = s1 + s2
        self.__d['PBS'] = str_pbs

    @property
    def pbw(self):
        """
        PBW (mode2ピッチ点の間隔[ms]) を取得
        例) PBW=77,163
        """
        # 辞書には文字列で登録してある
        s_pbw = self.__d['PBW']
        # 整数のリストに変換
        l_pbw = list(map(int, s_pbw.split(',')))
        # PBWの値をリストで返す
        return l_pbw

    @pbw.setter
    def pbw(self, list_pbw):
        """
        PBW (mode2ピッチ点の間隔[ms]) を登録
        例) PBW=77,163
        """
        # リストを整数の文字列に変換
        str_pbw = ','.join(list(map(str, map(int, list_pbw))))
        self.__d['PBW'] = str_pbw

    @property
    def pby(self):
        """
        PBY (mode2ピッチ点の高さ) を取得
        例) PBY=-10.6,0.0
        """
        # 辞書には文字列で登録してある
        s_pby = self.__d['PBY']
        # 整数のリストに変換
        l_pby = list(map(float, s_pby.split(',')))
        # PBYの値をリストで返す
        return l_pby

    @pby.setter
    def pby(self, list_pby):
        """
        PBY (mode2ピッチ点の高さ) を登録
        例) PBY=-10.6,0.0
        """
        # リストを小数の文字列に変換
        str_pby = ','.join(list(map(str, map(float, list_pby))))
        self.__d['PBY'] = str_pby

    @property
    def pbm(self):
        """
        PBM (mode2ピッチ点の形状) を取得
        例) PBY=-10.6,0.0
        """
        # 辞書には文字列で登録してある
        s_pby = self.__d['PBM']
        # 整数のリストに変換
        l_pbm = s_pby.split(',')
        # PBYの値をリストで返す
        return l_pbm

    @pbm.setter
    def pbm(self, list_pbm):
        """
        PBM (mode2ピッチ点の形状) を登録
        例) PBM=,r,j,s
        """
        # リストを文字列に変換
        str_pbm = ','.join(list_pbm)
        self.__d['PBM'] = str_pbm

    # ここからデータ操作系-----------------------------------------------------
    def get_by_key(self, key):
        """ノートの特定の情報を上書きまたは登録"""
        return self.__d[key]

    def set_by_key(self, key, x):
        """ノートの特定の情報を上書きまたは登録"""
        self.__d[key] = x
    # ここまでデータ操作系-----------------------------------------------------

    # ここからノート操作系-----------------------------------------------------
    def delete(self):
        """選択ノートを削除"""
        self.tag = '[#DELETE]'
        return self

    # def insert(self):
    #     """ノートを挿入(したい)"""
    #     self.tag = '[#INSERT]'
    #     return self

    def refresh(self):
        """
        ノートの情報を引き継ぎつつ、自由にいじれるようにする
        UTAUプラグインは値の上書きはできるが削除はできない。
        一旦ノートを削除して新規ノートとして扱う必要がある。
        """
        self.tag = '[#DELETE]\n[#INSERT]'

    def suppin(self):
        """ノートの情報を最小限にする"""
        new_note = {}
        new_note['Tag'] = '[#DELETE]\n[#INSERT]'
        new_note['Lyric'] = self.lyric
        new_note['Length'] = self.length
        new_note['NoteNum'] = self.notenum
        self.__d = new_note
    # ここまでノート操作系-----------------------------------------------------

    # ここからデータ出力系-----------------------------------------------------
    # Ustのほうで処理するようにしたので無効化しています。
    # -------------------------------------------------------------------------
    # def as_lines(self):
    #     """出力用のリストを返す"""
    #     d = self.__d
    #     lines = []
    #     lines.append(d.pop('Tag'))
    #     for k, v in d.items():
    #         line = '{}={}'.format(str(k), str(v))
    #         lines.append(line)
    #     return lines
    # ここまでデータ出力系-----------------------------------------------------


if __name__ == '__main__':
    main()

if __name__ == '__init__':
    pass



#!python3
# coding: utf-8
"""
SynthVのS5PとSVPを扱うモジュールです。
読み取り機能のみです。
"""
import json
from collections import OrderedDict


def load(path_svp):
    """
    SVPを読み取って辞書を返す
    """
    # ファイル内容を文字列として取得
    with open(path_svp) as f:
        s = f.read()
    # 最後の改行文字とかよくわからん記号を消す
    while not s.endswith('}'):
        s = s[:-1]
    # 歌詞とプロジェクト名がバイト列なため、ひらがなにする。
    b = s.encode()
    # 辞書にする(OrderedDictのほうがいいかなあ)
    d = json.loads(b)
    d = OrderedDict(d)
    # pprint(d, width=200)
    return d



#!python3
# coding: utf-8
# Copyright (c) oatsu
"""
REAPERのリージョンCSVを扱うモジュールです。
BOMなしUTF-8じゃないとREAPERは読み込んでくれません。
REAPER v5, v6 で動作確認

各行は "連番タグ,名前,開始時刻,終了時刻,長さ" になってる。
時刻は x.y.zz (小節.拍.割合(4/4拍子の時は8分音符:50, 16分音符:25))

時刻の zz は 4/4, 4/4 拍子の時は  8分音符:50 16分音符:25 の表記
                  6/8 拍子の時は 16分音符:50 32分音符:25 の表記

[例]---------------------------------------
  R1,_ああんいあう,5.1.00,9.1.00,4.0.00
  R2,_いいんういえ,9.1.00,13.1.00,4.0.00
-------------------------------------------

"""

import csv
from pprint import pprint


def main():
    print('---REAPERのリージョンCSVを扱うモジュールです。importして使ってね。---')
    print('CSV読み取りテストをします。')
    path = input('CSVのパスを入力してください。\n>>> ')
    l = load(path)
    print()
    pprint(l)


def load(path, mode='r', encoding='utf-8'):
    """
    REAPERのリージョンCSVを読み取る。
    """
    # 文字コードの候補
    choices = ['utf-8', 'shift-jis']
    if encoding in choices:
        choices.remove(encoding)

    # 指定された文字コードで読み取ろうとする
    try:
        with open(path, mode=mode, encoding=encoding) as f:
            reader = csv.reader(f)
            l = [row for row in reader]

    # 上手くいかなかったらもう片方の文字コードで読み取る
    except UnicodeDecodeError as e:
        print('[WARN]', e)
        print('[INFO] 文字コードを {} に変更して読み取ります。'.format(choices[0]))
        with open(path, mode=mode, encoding=choices[0]) as f:
            reader = csv.reader(f)
            l = [row for row in csv.reader(f)]

    tmp = []
    # 行ごとにRegionオブジェクトを生成
    keys = tuple(l[0])
    for v in l[1:]:
        region = Region()
        region.values = dict(zip(keys, v))
        tmp.append(region)

    regioncsv = RegionCsv()
    regioncsv.values = tmp
    return regioncsv


class RegionCsv:
    """
    REAPERのリージョンCSV用のクラス
    """

    def __init__(self):
        # 辞書からなるリスト
        self.__l = []

    @property
    def values(self):
        """
        値を確認
        """
        return self.__l

    @values.setter
    def values(self, l):
        """
        値を代入
        """
        self.__l = l


    def append(self, region):
        """Regionオブジェクトを末尾に追加"""
        self.__l.append(region)


    def write(self, path, mode='w'):
        """
        REAPERのリージョンに適したCSVを出力する。
        """
        rows = [['#', 'Name', 'Start', 'End', 'Length']]
        for i, region in enumerate(self.__l):
            row = [f'R{i}', region.name, region.start, region.end, region.length]
            rows.append(row)
        # ファイル出力
        with open(path, mode=mode, encoding='utf-8', newline='\n') as f:
            writer = csv.writer(f)
            writer.writerows(rows)


class Region:
    """
    REAPERのリージョンひとつ分のクラス
    辞書
    """

    def __init__(self):
        self.__d = {'#': '', 'Name': '', 'Start': '', 'End': '', 'Length': ''}

    @property
    def values(self):
        """
        値を確認
        """
        return self.__d

    @values.setter
    def values(self, d):
        """
        値を代入
        """
        self.__d = d

    @property
    def name(self):
        return self.__d['Name']

    @name.setter
    def name(self, x):
        self.__d['Name'] = x

    @property
    def start(self):
        return self.__d['Start']

    @start.setter
    def start(self, x):
        self.__d['Start'] = x

    @property
    def end(self):
        return self.__d['End']

    @end.setter
    def end(self, x):
        self.__d['End'] = x

    @property
    def length(self):
        return self.__d['Length']

    @length.setter
    def length(self, x):
        self.__d['Length'] = x


if __name__ == '__main__':
    main()
    print('Press enter to exit.')



#!python3
# coding: utf-8
"""
SHIRO (https://github.com/Sleepwalking/SHIRO) 関連のファイルを扱う。
"""

import re


def load_index(path, mode='r', encoding='shift-jis'):
    """index fileを読み取ってIndexを返す"""
    # labファイル読み取り
    with open(path, mode=mode, encoding=encoding) as f:
        lines = [re.split(', ') for s in f.readlines()]
    idx = Index()
    idx.values = lines
    return idx


class Index:
    """
    wavに含まれる音素一覧を提示する 'index file' を扱うクラス。
        音声ファイル名1,音素1-1 音素1-2 音素1-3 音素1-4 ...
        音声ファイル名2,音素2-1 音素2-2 音素3-3 音素4-4 ...
        ...
    ってフォーマットのファイル。
    """

    def __init__(self):
        """二次元リスト [[開始時刻, 音素, 音素, 音素, ...], [], ...]"""
        self.__values = []

    def write(self, path, mode='w', encoding='shift-jis', newline='\n'):
        """indexfile(CSV)を保存"""
        l = self.values
        s = ''
        for v in l:
            s += '{},{}\n'.format(v[0], ' '.join(v[1:]))
        # ファイル出力
        with open(path, mode=mode, encoding=encoding, newline=newline) as f:
            f.write(s)
        return s

    @property
    def values(self):
        """値を取得"""
        return self.__values

    @values.setter
    def values(self, lines):
        """値を登録"""
        if not isinstance(lines, list):
            raise TypeError('\"lines\" must be list instance (values.setter in label.py)')
        self.__values = lines



#!python
# coding: utf-8
# Copyright (c) oatsu
"""
Python script for PyPI registation
"""
from setuptools import find_packages, setup

version = '1.6.5'

try:
    with open("README.md", mode='r') as f:
        long_description = f.read()
except UnicodeDecodeError:
    with open("README.md", mode='r', encoding='utf-8') as f:
        long_description = f.read()

setup(
    name='utaupy',
    version=version,
    description='Python3 module for UTAU and singing-voice-database',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='oatsu',
    author_email='panchi.psp@gmail.com',
    maintainer='oatsu',
    maintainer_email='panchi.psp@gmail.com',
    url='https://github.com/oatsu-gh/utaupy',
    packages=find_packages(),
    install_requires=[],
    classifiers=[
        'Programming Language :: Python :: 3'
    ],
    keywords=['UTAU']
)



#!python3
# coding: utf-8
"""
UTAUの録音リストを扱うモジュール
録音リストは休憩する部分で空白行を入れてね
"""
import re
from pprint import pprint


def main():
    """直接実行されたときの動作"""
    print('---録音リストを扱うためのモジュールです。importして使ってね。---')
    print('録音リスト読み取り動作チェックをします。')
    # pathを標準入力
    path_reclist = input('録音リストのパスを入力してください。\n>>> ')
    # OREMOのコメントの扱いを質問
    oremo = input('OREMO用のコメントを含んでいますか？(Y/n)\n>>> ') in ['Y', 'y', 'Ｙ', 'ｙ']
    # 録音リストの空行の扱いを質問
    remove_blankline = input('空行を削除しますか？(Y/n)\n>>> ') in ['Y', 'y', 'Ｙ', 'ｙ']
    # 録音リストを読み取る
    l = load(path_reclist, remove_blankline=remove_blankline, oremo=oremo)
    print('\n取得結果---------------------------')
    pprint(l)
    print('-----------------------------------')


def load(path, mode='r', encoding='shift-jis', remove_blankline=True, oremo=False):
    """
    録音リストを読み取ってオブジェクト生成
    path    : 録音リストのパス
    mode    : ファイルをopen()するときののモード
    rmblank : 空行を削除するかどうか
    oremo   : 録音ソフトウェア「OREMO」用のコメントの有無。あるならTrue
    """
    # 録音リストのデータ用リスト
    l = []

    # OREMOのコメントを無視する
    if oremo:
        with open(path, mode=mode, encoding=encoding) as f:
            l = [line.strip().split()[0] for line in f.readlines()]
    # OREMOのコメントを無視しないが、全角スペースで分割する
    else:
        with open(path, mode=mode, encoding=encoding) as f:
            for line in f.readlines():
                l += re.split('[　 ]', line.strip())
    # 空行を削除する
    if remove_blankline:
        l = [v for v in l if v != '']

    return l


if __name__ == '__main__':
    main()
    input('Press Enter to exit.')
